﻿package ru.flexfactory
{
	import flash.display.*;
	import flash.net.URLRequest;
	import flash.events.*;
	import flash.utils.Timer;

	import com.greensock.*;
	import com.greensock.easing.*;
	import com.greensock.plugins.*;
	import flash.geom.Point;

	public class Slider extends MovieClip
	{
		private var xmlData:XML;            //загруженный XML-файл
		private var countSlide:int = 0;     //число изобажений
		private var slide:Slide;            //переменная для ссылки на слайды
		private var slides:Array;           //массив слайдов
		private var timer:Timer;            //таймер для перебора слайдов
		private var i:int;                  
		private var options:Options;        //объект класса Options для хранения опций
		private var points:AlignPoint;      //объект класса AlignPoint для получения случайной точки

		//конструктор, в который мы передаём загруженный извне файл XML с данными
		public function Slider(xml:XML)
		{
			super();
			xmlData = xml;
			countSlide = xmlData.slide.length();
			slides = new Array(countSlide);
			options = new Options();
		}

		//метод для подготовки объекта и запуска таймера
		public function init():void
		{
			if (countSlide == 0) return;

			TweenPlugin.activate([TransformAroundPointPlugin]); 
			setOption();

			timer = new Timer(options.durationSlide * 1000);
			timer.addEventListener(TimerEvent.TIMER, timerHandler);
			
			points = new AlignPoint(stage.stageWidth,stage.stageHeight);

			i = 0;
			
			start();
		}

		//метод для сохранения входных параметров в объекте options
		//на самом деле можно было обойтись и без этого класса и просто обращаться к объекту xml, но
		//используя данный класс, нам не нужно будет вспоминать всю структуру xml файла каждый раз 
		//при необходимости, следовательно меньше вероятность допустить ошибку
		private function setOption():void
		{
			options.durationSlide = int(xmlData.options.duration_slide);
			options.durationFade = int(xmlData.options.duration_fade);
			options.minScale = Number(xmlData.options.min_scale);
			options.maxScale = Number(xmlData.options.max_scale);
		}

		//запускаем механизм
		private function start():void
		{
			timer.start();
			timerHandler(null);
		}

		//функция, которая выполняется через определённый в таймере промежуток времени
		private function timerHandler(event:TimerEvent):void
		{
			//берём объект из массива
			slide = slides[i];
			
			//если он не загружен, то загружаем и переходим к обработке
			if (slide == null)  
			{
				slide = new Slide(xmlData.*[i].path, options.durationFade);
				slides[i] = slide;
				slide.addEventListener(Event.COMPLETE, slideCompleteHandler);
			}
			//если загружен, то просто обрабатываем
			else 
			{
				slideCompleteHandler(null);
			}
		}

		//применяем эффект масштабирования
		private function slideCompleteHandler(event:Event):void
		{
			//если мы пришли в эту функцию через слушателя, то удаляем его
			if (event != null) event.target.removeEventListener(Event.COMPLETE, slideCompleteHandler);
			
			//масштаб по-умолчанию
			slide.scaleX = 1;
			slide.scaleY = 1;
			
			//если слайд ещё не добавлен в слайдер, то добавляем
			addIfOut();
			
			//выносим текущий слайд поверх остальных
			setChildIndex(slide, numChildren - 1);
			
			//запускаем эффект появления
			slide.fadeIn();
			
			//берём произвольную точку для выравнивания
			var point:Point = points.randomPoint;
			
			//выравниваем мувик относительно этой точки
			alignByPoint(slide, point);

			//этот кусок кода можно написать через if(), но тогда параметры метода нужно будет писать 2 раза
			//"правильное" программирование исключает копипаст, поэтому сделаем следующим образом
			//присваиваем переменной action метод, который выбираем случайно 50/50. 
			var action:Function = (Math.random() * 2 < 1) ? TweenLite.from : TweenLite.to;
			
			//выбираем произвольное значение масштабирования в пределах от minScale до maxScale
			var scale:Number = (Math.random() * (options.maxScale - options.minScale)) + options.minScale;
			
			//выполняем выбранный метод TweenLite
			action(slide, 
				   options.durationSlide + options.durationFade, 
				   {transformAroundPoint:{point:point, scaleX:scale, scaleY:scale}, 
				   ease:Linear.easeNone});

			//увеличиваем либо сбрасываем значение счётчика
			if (++i == countSlide) i = 0;
		}
		
		//если слайд ещё не добавлен в слайдер, то добавляем
		private function addIfOut():void
		{
			for (var j:int = 0; j < numChildren; j++)
			{
				if (slide == getChildAt(j)) return;
			}
			addChildAt(slide, 0);
		}
	
		//метод для выравнивания изображения относительно точки
		private function alignByPoint(slide:Slide, point:Point):void
		{
			if (point.x == 0) slide.x = 0;
			if (point.x == stage.stageWidth / 2) slide.x = -(slide.width - stage.stageWidth) / 2;
			if (point.x == stage.stageWidth) slide.x = -(slide.width - stage.stageWidth);

			if (point.y == 0) slide.y = 0;
			if (point.y == stage.stageHeight / 2) slide.y = -(slide.height - stage.stageHeight) / 2;
			if (point.y == stage.stageHeight) slide.y = -(slide.height - stage.stageHeight);
		}

		private function ioErrorHandler(event:IOErrorEvent):void
		{
			trace(event.text);
		}
		
		//метод для перехода на следующий слайд
		public function showNext():void
		{
			if (timer == null) return;   
			
			timer.reset();
			timer.start();
			timerHandler(null);
		}
		
		//метод для перехода на предыдущий слайд
		public function showPrev():void
		{
			if (timer == null) return;
			
			i -= 2;
			if (i < 0) i += countSlide;
			
			showNext();
		}
	}
}